#include "framecpp/config.h"

#include <algorithm>

#include "framecpp/Common/IOStream.hh"
#include "framecpp/Common/Description.hh"

#include "framecpp/Version6/FrStatData.hh"
#include "framecpp/Version6/FrDetector.hh"
#include "framecpp/Version6/FrSE.hh"
#include "framecpp/Version6/FrSH.hh"
#include "framecpp/Version6/PTR_STRUCT.hh"

using namespace FrameCPP::Version_6;
using FrameCPP::Common::Description;
using FrameCPP::Common::FrameSpec;
using FrameCPP::Common::IStream;
using FrameCPP::Common::OStream;

//=======================================================================
// FrStatData
//=======================================================================

FrStatData::
FrStatData( )
  : Common::FrStatData( StructDescription( ) )
{
}

FrStatData::
FrStatData( const FrStatData& Source )
  : Common::FrStatData( StructDescription( ) ),
    FrStatDataNPS( Source ),
    FrStatDataPS( Source )
{
}

FrStatData::
FrStatData( const std::string& name,
	    const std::string& comment,
	    const std::string& representation,
	    const INT_4U timeStart,
	    const INT_4U timeEnd,
	    const INT_4U version )
  : Common::FrStatData( StructDescription( ) ),
    FrStatDataNPS( name, comment, representation,
		   timeStart, timeEnd, version )
{
}

FrStatData::
FrStatData( const Previous::FrStatData& Source,
	    Common::IStream* Stream )
  : Common::FrStatData( StructDescription( ) )
{
  m_data( Source );
  if ( Stream )
  {
    //-------------------------------------------------------------------
    // Modify references
    //-----------------------------------------------------------------
    const INT_2U max_ref = Previous::FrStatData::MAX_REF;
    Stream->ReplacePtr( (FrameSpec::Object**)(AddressOfDetector( )),
			(const FrameSpec::Object**)(Source.AddressOfDetector( )),
			max_ref );
    Stream->ReplaceRef( RefData( ), &(Source.RefData( )), max_ref );
    Stream->ReplaceRef( RefTable( ), &(Source.RefTable( )), max_ref );
  }
}

FrStatData::
FrStatData( IStream& Stream )
  : Common::FrStatData( StructDescription( ) )
{
  m_data( Stream );
  m_refs( Stream );
}

FrameCPP::cmn_streamsize_type FrStatData::
Bytes( const Common::StreamBase& Stream ) const
{
  return
    m_data.Bytes( )
    + m_refs.Bytes( Stream )
    ;
}

FrStatData* FrStatData::
Clone( ) const
{
  return new FrStatData( *this );
}

FrStatData* FrStatData::
Create( IStream& Stream ) const
{
  return new FrStatData( Stream );
}

const char* const FrStatData::
ObjectStructName( ) const
{
  return StructName( );
}

const Description* FrStatData::
StructDescription( )
{
  static Description ret;

  if ( ret.size( ) == 0 )
  {
    ret( FrSH( FrStatData::StructName( ), FrStatData::STRUCT_ID,
	       "Static Data Structure" ) );

    FrStatDataNPS::data_type::Describe< FrSE >( ret );
    ref_type::Describe< FrSE >( ret );
  }

  return &ret;
}

void FrStatData::
Write( OStream& Stream ) const
{
  m_data( Stream );
  m_refs( Stream );
}

void FrStatData::
TOCQuery( int InfoClass, ... ) const
{
  using Common::TOCInfo;

  va_list	vl;
  va_start( vl, InfoClass );
  while ( InfoClass != TOCInfo::IC_EOQ )
  {
    int data_type = va_arg( vl, int );
    switch( data_type )
    {
    case TOCInfo::DT_STRING_2:
      {
	STRING* data = va_arg( vl, STRING* );
	switch( InfoClass )
	{
	case TOCInfo::IC_NAME:
	  *data = GetName( );
	  break;
	case TOCInfo::IC_DETECTOR:
	  if ( GetDetector( ) )
	  {
	    *data = GetDetector( )->GetName( );
	  }
	  else
	  {
	    *data = "";
	  }
	  break;
	default:
	  goto cleanup;
	  break;
	}
      }
      break;
    case TOCInfo::DT_INT_4U:
      {
	INT_4U* data = va_arg( vl, INT_4U* );
	switch( InfoClass )
	{
	case TOCInfo::IC_START:
	  *data = GetTimeStart( );
	  break;
	case TOCInfo::IC_END:
	  *data = GetTimeEnd( );
	  break;
	case TOCInfo::IC_VERSION:
	  *data = GetVersion( );
	  break;
	default:
	  goto cleanup;
	  break;
	}
      }
      break;
    default:
      // Stop processing
      goto cleanup;
    }
    InfoClass = va_arg( vl, int );
  }
 cleanup:
  va_end( vl );
}

FrStatData& FrStatData::
Merge( const FrStatData& RHS )
{
  //:TODO: Need to implement Merge routine
  std::string msg( "Merge currently not implemented for " );
  msg += StructName( );

  throw std::domain_error( msg );
  return *this;
}


bool FrStatData::
operator==( const Common::FrameSpec::Object& Obj ) const
{
  return ( *this == Obj );
}

std::auto_ptr< FrameSpec::Object > FrStatData::
demote( INT_2U Target, IStream* Stream ) const
{
  if ( Target >= DATA_FORMAT_VERSION )
  {
    return std::auto_ptr< FrameSpec::Object >
      ( const_cast< FrStatData* >( this ) );
  }
  try
  {
    //-------------------------------------------------------------------
    // Copy non-reference information
    //-------------------------------------------------------------------
    // Do actual down conversion
    std::auto_ptr< Previous::FrStatData >
      retval( new Previous::FrStatData( GetName( ),
					GetComment( ),
					GetRepresentation( ),
					GetTimeStart( ),
					GetTimeEnd( ),
					GetVersion( )
					) )
      ;
    if ( Stream )
    {
      //-----------------------------------------------------------------
      // Modify references
      //-----------------------------------------------------------------
      Stream->ReplacePtr( (FrameSpec::Object**)(retval->AddressOfDetector( )),
			  (const FrameSpec::Object**)(AddressOfDetector( )), MAX_REF );
      Stream->ReplaceRef( retval->RefData( ), &(RefData( )), MAX_REF );
      Stream->ReplaceRef( retval->RefTable( ), &(RefTable( ) ), MAX_REF );
    }
    //-------------------------------------------------------------------
    // Return demoted object
    //-------------------------------------------------------------------
    return std::auto_ptr< FrameSpec::Object >( retval.release( ) );
  }
  catch( ... )
  {
  }
  throw
    Unimplemented( "Object* FrStatData::demote( Object* Obj ) const",
		   DATA_FORMAT_VERSION, __FILE__, __LINE__ );
}

std::auto_ptr< FrameSpec::Object > FrStatData::
promote( INT_2U Target, Object* Obj, IStream* Stream ) const
{
  return Promote( Target, Obj, Stream );
}
